UNIT MINIPX; (* AUTOR: R. Gilomen - ᯠᨡ, Danke     *)
             (* Example       V. Chronusov 2:5054/4.11 *)
             (*   ࠡ⠥  ⢠⮬ *)
             (*   ᥬ 設 ࠧ *)
INTERFACE

CONST
       MySocket             = $4343; (*    IPX ( ᬥ,pls)*)
       MAX_DATA_SIZE        = 546;
(* Code Deklarationen *)
       NOT_ENDED             = $10;

TYPE
       Data_Packet     = ARRAY [1..MAX_DATA_SIZE] OF CHAR;
       SData           = RECORD
                           Data   : Data_Packet;
                           Length : WORD;
                          END;
VAR
 ReceiveEvent:boolean; (*   -    !!!*)

{-------------------}
FUNCTION IPX_Init : Boolean;
Procedure IPX_Close_Socket;
FUNCTION IPX_Send ( Buffer : SData ) : BYTE;
FUNCTION IPX_Receive : BYTE;
FUNCTION IPX_DoSend ( VAR Buffer : SData ) : BYTE;
FUNCTION IPX_DoReceive ( VAR Buffer : SData ) : BYTE;
{-------------------}

IMPLEMENTATION
USES Dos;
CONST
(* Allgemeine Deklarationen *)
         NET_LENGTH           = 4;     (* Laenge Netzwerkadresse            *)
         NODE_LENGTH          = 6;     (* Laenge Knotenadresse              *)
(* Deklaration der Rueckgabewerte *)
         SUCCESS               = $00;
         PARAMETER_ERROR       = $20;
         NO_DESTINATION        = $21;
         DEVICE_SW_ERROR       = $30;
         SOCKET_TABLE_FULL     = $31;
         PACKET_BAD            = $32;
         PACKET_UNDELIVERIABLE = $33;
         PACKET_OVERFLOW       = $34;
         DEVICE_HW_ERROR       = $40;

TYPE   S4Byte          =  ARRAY [1..4]  OF BYTE; (* Datentyp fuer Network   *)
       S6Byte          =  ARRAY [1..6]  OF BYTE; (* Datentyp fuer Node      *)
                                                 (* Datentyp fuer Daten     *)
       Network_Address = RECORD                  (* Datentyp fuer NW-Adr.   *)
                           Network     : S4Byte;
                           Node        : S6Byte;
                           Socket      : WORD;
                         END;

CONST
(* Allgemeine Definitionen *)
         HEADER       = 30;            (* Groesse IPX-Header                *)
         PACKET_SIZE  = 576;           (* IPX-Paket groesse                 *)
(* Definitionen der IPX-Funktionen *)
         IPX_TST      = $7A00;         (* Vorbereiten fuer IPX Test         *)
         MUX_INTR     = $2F;           (* Multiplex Interrupt               *)
         OPEN_SOCKET  = $0000;         (* Oeffnet einen Sockel              *)
         CLOSE_SOCKET = $0001;         (* Schliesst einen Sockel            *)
         GET_TARGET   = $0002;         (* Pruefe Gegenstation               *)
         DO_SEND      = $0003;         (* Sendet ein Paket                  *)
         DO_RECEIVE   = $0004;         (* Empfaengt Pakete                  *)
         GET_ADDR     = $0009;         (* Bestimmt Internetzwerkadresse     *)
(* Definitionen der IPX-Parameter *)
         STAY_OPEN    = $FF;           (* $00 : Sockel bleibt geoeffnet,    *)
                                       (* bis er explizit geschlossen wird  *)
                                       (* oder das Programm terminiert.     *)
                                       (* $FF : Sockel bleibt geoeffnet,    *)
                                       (* bis er explizit geschlossen wird. *)
                                       (* Wird benoetigt fuer TSR-Programme.*)
(* Definitionen der IPX-Rueckgabewerte *)
         IPX_LOADED   = $FF;           (* IPX ist geladen                   *)
         OPENED       = $00;           (* Sockel erfolgreich geoeffnet      *)
         ALREADY_OPEN = $FF;           (* Sockel ist bereits goeffnet       *)
         TABLE_FULL   = $FE;           (* Sockel Tabelle ist voll           *)
         EXIST        = $00;           (* Weg zu Gegenstation existiert     *)
         NO_SOCKET    = $FF;           (* Sockel existiert nicht            *)
         SEND_OK      = $00;           (* Senden war erfolgreich            *)
         SOCKET_ERROR = $FC;           (* Sockel existiert nicht mehr       *)
         SIZE_ERROR   = $FD;           (* Paketgroesse nicht korrekt        *)
         UNDELIV      = $FE;           (* Paket nicht ausgeliefert          *)
         OVERFLOW     = $FD;           (* Buffer zu klein                   *)
         HW_ERROR     = $FF;           (* Hardware defekt                   *)
         REC_OK       = $00;           (* Paket erfolgreich empfangen       *)
(* Definition der ECB-Parameter *)
         FINISHED     = $00;           (* Routine beendet                   *)
         FRAG_COUNT   = 1;             (* Anzahl Fragmente                  *)
         UNKNOWN      = 0;             (* Unbekannter Paket Typ             *)

(* Deklarationen *)
TYPE     S12Byte      = ARRAY [1..12] OF BYTE;   (* Interner Datentyp       *)
         IPX_Packet   = RECORD         (* IPX-Paket Struktur                *)
                          CheckSum         : WORD;
                          Length           : WORD;
                          TransportControl : BYTE;
                          PacketType       : BYTE;
                          Destination      : Network_Address;
                          Source           : Network_Address;
                          IPX_Data         : Data_Packet;
                        END;
         ECB_Fragment = RECORD         (* Fragment der ECB Struktur         *)
                          Address : ^IPX_Packet;
                          Size    : WORD;
                        END;
         ECB = RECORD                  (* ECB Datenstruktur                 *)
                Link_Adress        : S4Byte;
                ESR_Address        : ^BYTE;   { 뫪  ESR }
                InUseFlag          : BYTE;
                CompletionCode     : BYTE;
                SocketNumber       : WORD;
                IPX_Workspace      : S4Byte;
                DriverWorkspace    : S12Byte;
                ImmediateAddress   : S6Byte;
                FragmentCount      : WORD;
                FragDescr          : ECB_Fragment;
               END;
         Int_Addr = RECORD             (* Datenstruktur Internetzwerkadr.   *)
                      Network : S4Byte;
                      Node    : S6Byte;
                    END;

VAR      IPX_Location : ARRAY [1..2] OF WORD;    (* Adresse von IPX         *)
                                                 (* Array in dem die ECB's  *)
                                                 (* verwaltet werden.       *)
         ECB_Table    : ECB;


{-------------------}

PROCEDURE IPX_Call ( VAR Regs : Registers );
VAR Temp_AX, Temp_BX, Temp_CX, Temp_DX,
    Temp_ES, Temp_SI, Temp_DI : WORD;
BEGIN
 Temp_AX := Regs.AX;
 Temp_BX := Regs.BX;
 Temp_CX := Regs.CX;
 Temp_DX := Regs.DX;
 Temp_SI := Regs.SI;
 Temp_ES := Regs.ES;
 Temp_DI := Regs.DI;
 ASM
  PUSH BP                              (* Register sichern                  *)
  PUSH SP
  PUSH SS
  PUSH DS
  PUSH AX
  PUSH BX
  PUSH CX
  PUSH DX
  PUSH SI
  PUSH ES
  PUSH DI
  MOV AX, Temp_AX                      (* Register setzen                   *)
  MOV BX, Temp_BX
  MOV CX, Temp_CX
  MOV DX, Temp_DX
  MOV SI, Temp_SI
  MOV ES, Temp_ES
  MOV DI, Temp_DI
  CALL DWORD PTR IPX_Location          (* IPX aufrufen                      *)
  MOV Temp_AX, AX                      (* Register auslesen                 *)
  MOV Temp_BX, BX
  MOV Temp_CX, CX
  MOV Temp_DX, DX
  MOV Temp_SI, SI
  MOV Temp_ES, ES
  MOV Temp_DI, DI
  POP DI
  POP ES                               (* Gesicherte Register wieder        *)
  POP SI                               (* zuruecksetzen.                    *)
  POP DX
  POP CX
  POP BX
  POP AX
  POP DS
  POP SS
  POP SP
  POP BP
 END;
 Regs.AX := Temp_AX;
 Regs.BX := Temp_BX;
 Regs.CX := Temp_CX;
 Regs.DX := Temp_DX;
 Regs.SI := Temp_SI;
 Regs.ES := Temp_ES;
 Regs.DI := Temp_DI;
END; { IPX_Call }

{-------------------}

Procedure ESR_Receive;far;
begin
 asm {  ᮢ   }
  mov ax,es
  mov ds,ax
 end;
 ReceiveEvent:=true;
end;

{-------------------}

FUNCTION IPX_Init : Boolean;
VAR  Temp_Reg : Registers;         (* Temporaere Register fuer IPX-Call *)
     Socket : WORD;
BEGIN
  IPX_Init:=false;
  Temp_Reg.AX := IPX_TST;              (* Test ob IPX geladen.              *)
  Intr (MUX_INTR,Temp_Reg);
  IF (Temp_Reg.AL <> IPX_LOADED) THEN EXIT;
  Temp_Reg.AX := Temp_Reg.ES;
  IPX_Location[1] := Temp_Reg.DI;      (* Adresse von IPX sichern           *)
  IPX_Location[2] := Temp_Reg.AX;

  Socket:=MySocket;
  Socket := Swap(Socket);              (* In Motorola Format konvertieren   *)
  Temp_Reg.BX := OPEN_SOCKET;          (* Register fuer Call vorbereiten    *)
  Temp_Reg.AL := STAY_OPEN;
  Temp_Reg.DX := Socket;
  IPX_Call (Temp_Reg);
  Socket := Temp_Reg.DX;               (* Register auslesen                 *)
  IF Temp_Reg.AL <> OPENED THEN EXIT;
  NEW (ECB_Table.FragDescr.Address);
  ECB_Table.SocketNumber := Socket;
  IPX_Init := true;
END; { IPX_Open_Socket }

{-------------------}

Procedure IPX_Close_Socket;
VAR      Temp_Reg : Registers;         (* Temporaere Register fuer IPX-Call *)
         Socket : WORD;
BEGIN
  Socket:=MySocket;
  Socket := Swap(Socket);              (* In Motorola Format konvertieren   *)
  Temp_Reg.BX := CLOSE_SOCKET;         (* Register fuer Call vorbereiten    *)
  Temp_Reg.DX := Socket;
  IPX_Call (Temp_Reg);
  DISPOSE(ECB_Table.FragDescr.Address);
  ECB_Table.FragDescr.Address := NIL;
END; { IPX_Close_Socket }

{-------------------}

FUNCTION IPX_Send ( Buffer : SData ) : BYTE;
VAR  Temp_Reg  : Registers;        (* Temporaere Register fuer IPX-Call *)
     Temp_Imm_Addr : S6Byte;       (* Temporaere ImmdediateAddress      *)
     Temp_Addr : S12Byte;          (* Temporaere Internetworkadresse    *)
     Socket : WORD;
     Dest_Addr : Network_Address;
     i:integer;
BEGIN
  Socket:=MySocket;
  Dest_Addr.Socket := MySocket;
  WITH Dest_Addr DO
  BEGIN
   fillchar(Network,4,0);
   fillchar(Node,6,$ff);
  END;
  Socket := Swap(Socket);              (* In Motorola Format konvertieren   *)
  Dest_Addr.Socket := Swap(Dest_Addr.Socket);
  IF Buffer.Length > MAX_DATA_SIZE THEN     (* Laenge der Daten pruefen     *)
  BEGIN
    IPX_Send := PARAMETER_ERROR;
    EXIT;
  END;
  WITH Dest_Addr DO                    (* Pruefe ob Gegenstation erreichbar *)
  BEGIN
    FOR i := 1 TO NET_LENGTH DO        (* Internetzwerkadresse zusammenst.  *)
      Temp_Addr[i] := Network[i];
    FOR i := 1 TO NODE_LENGTH DO
      Temp_Addr[i + NET_LENGTH] := Node[i];
    Temp_Addr[11] := Lo(Socket);
    Temp_Addr[12] := HI(Socket);
  END;
  Temp_Reg.ES := Seg(Temp_Addr);       (* Register fuer Call vorbereiten    *)
  Temp_Reg.SI := Ofs(Temp_Addr);
  Temp_Reg.DI := Ofs(Temp_Imm_Addr);
  Temp_Reg.BX := GET_TARGET;
  IPX_Call(Temp_Reg);
  ECB_Table.ImmediateAddress := Temp_Imm_Addr;
  IF Temp_Reg.AL <> EXIST THEN
  BEGIN
    IPX_Send := NO_DESTINATION;        (* Weg nicht verfuegbar              *)
    EXIT;
  END;
  WITH ECB_Table DO            (* ECB mit Parametern fuellen        *)
  BEGIN
    ESR_Address := NIL;               { 뫪  ESR ^Byte }
    SocketNumber := Socket;
    InUseFlag := FINISHED;
    FragmentCount := FRAG_COUNT;
    WITH FragDescr.Address^ DO         (* IPX-Header vorbereiten            *)
    BEGIN
      PacketType := UNKNOWN;
      Destination:=Dest_Addr;
      IPX_Data := Buffer.Data;
    END;
    FragDescr.Size := Buffer.Length + 30;
  END;
  Temp_Reg.ES := Seg(ECB_Table);  (* Register fuer Call vorbereiten *)
  Temp_Reg.SI := Ofs(ECB_Table);
  Temp_Reg.BX := DO_SEND;
  IPX_Call (Temp_Reg);
  IPX_Send := SUCCESS;
END; { IPX_Send }

{-------------------}

FUNCTION IPX_Receive : BYTE;
VAR Temp_Reg  : Registers;        (* Temporaere Register fuer IPX-Call *)
BEGIN
  WITH ECB_Table DO            (* ECB mit Parametern fuellen        *)
  BEGIN
    ESR_Address := @ESR_Receive ;     { 뫪  ESR ^Byte }
    FragmentCount := FRAG_COUNT;
    FragDescr.Size := PACKET_SIZE;
    InUseFlag := FINISHED;
  END;
  Temp_Reg.ES := Seg(ECB_Table);    (* Register vorbereiten         *)
  Temp_Reg.SI := Ofs(ECB_Table);
  Temp_Reg.BX := DO_RECEIVE;
  IPX_Call (Temp_Reg);
  IF Temp_Reg.AL = NO_SOCKET THEN
  BEGIN
    IPX_Receive := DEVICE_SW_ERROR;
    EXIT;
  END;
  IPX_Receive := SUCCESS;
END; { IPX_Receive }

{-------------------}

FUNCTION IPX_DoSend ( VAR Buffer : SData ) : BYTE;
BEGIN

 IF ECB_Table.InUseFlag <> FINISHED THEN
 BEGIN
  IPX_DoSend := NOT_ENDED;
  EXIT;
 END;

 CASE ECB_Table.CompletionCode OF
  SEND_OK      : IPX_DoSend := SUCCESS;
  SOCKET_ERROR : IPX_DoSend := DEVICE_SW_ERROR;
  SIZE_ERROR   : IPX_DoSend := PACKET_BAD;
  UNDELIV      : IPX_DoSend := PACKET_UNDELIVERIABLE;
  HW_ERROR     : IPX_DoSend := DEVICE_HW_ERROR;
  ELSE           IPX_DoSend := DEVICE_SW_ERROR;
 END; { CASE ECB_Table.CompletionCode , SEND }

END;

{-------------------}

FUNCTION IPX_DoReceive ( VAR Buffer : SData ) : BYTE;
BEGIN

 IF ECB_Table.InUseFlag <> FINISHED THEN
 BEGIN
  IPX_DoReceive := NOT_ENDED;
  EXIT;
 END;

 CASE ECB_Table.CompletionCode OF
  REC_OK       : BEGIN
                  Buffer.Data   :=
                   ECB_Table.FragDescr.Address^.IPX_Data;
                  Buffer.Length :=
                   Swap(ECB_Table.FragDescr.Address^.Length) - HEADER;
                  IPX_DoReceive := SUCCESS;

                  ReceiveEvent:=false; {  ⥢ ᮡ }

                 END; { REC_OK }
  SOCKET_ERROR : IPX_DoReceive := DEVICE_SW_ERROR;
  OVERFLOW     : IPX_DoReceive := PACKET_OVERFLOW;
  NO_SOCKET    : IPX_DoReceive := DEVICE_SW_ERROR;
  ELSE           IPX_DoReceive := DEVICE_SW_ERROR;
 END { CASE ECB_Table.CompletionCode ,receive }

END;

begin
 ReceiveEvent:=false;
END.

